@@ -270,6 +270,11 @@ def parseconfig(args, plugins=()):
         pm.hook.tox_configure(config=config)  # post process config object
         break
     else:
+        parser = Parser()
+        pm.hook.tox_addoption(parser=parser)
+        # if no tox config file, now we need do a strict argument evaluation
+        # raise on unknown args
+        parser.parse_cli(args, strict=True)
         if option.help or option.helpini:
             return config
         msg = "tox config file (either {}) not found"
@@ -683,6 +688,7 @@ def tox_addoption(parser):
             "LD_LIBRARY_PATH",
             "PATH",
             "PIP_INDEX_URL",
+            "PIP_EXTRA_INDEX_URL",
             "REQUESTS_CA_BUNDLE",
             "SSL_CERT_FILE",
             "TOX_WORK_DIR",
@@ -1010,6 +1016,8 @@ class ParseIni(object):
         self._cfg = py.iniconfig.IniConfig(config.toxinipath, ini_data)
         previous_line_of = self._cfg.lineof
 
+        self.expand_section_names(self._cfg)
+
         def line_of_default_to_zero(section, name=None):
             at = previous_line_of(section, name=name)
             if at is None:
@@ -1348,6 +1356,28 @@ class ParseIni(object):
             raise tox.exception.ConfigError(msg)
         return env_list, all_envs, _split_env(from_config), envlist_explicit
 
+    @staticmethod
+    def expand_section_names(config):
+        """Generative section names.
+
+        Allow writing section as [testenv:py{36,37}-cov]
+        The parser will see it as two different sections: [testenv:py36-cov], [testenv:py37-cov]
+
+        """
+        factor_re = re.compile(r"\{\s*([\w\s,]+)\s*\}")
+        split_re = re.compile(r"\s*,\s*")
+        to_remove = set()
+        for section in list(config.sections):
+            split_section = factor_re.split(section)
+            for parts in itertools.product(*map(split_re.split, split_section)):
+                section_name = "".join(parts)
+                if section_name not in config.sections:
+                    config.sections[section_name] = config.sections[section]
+                    to_remove.add(section)
+
+        for section in to_remove:
+            del config.sections[section]
+
 
 def _split_env(env):
     """if handed a list, action="append" was used for -e """
